#!/usr/local/bin/perl -w

# This script takes actmap action compare files and averages the values

$usage = "$0 <list of actmap action compare files>";

die $usage if not (@ARGV);

use Statistics::Descriptive;

$cmdline = join ' ', ($0, @ARGV);
$pwd = `pwd`;
chomp $pwd;

while (<>)
  {
    s/\#.*//;
    next if (/^\s*$/);
    ($time, $numpoints, $val) = split;
    next if ($numpoints == 0);
    if (not exists $timevals{$time})
      {
        $timevals{$time} = Statistics::Descriptive::Sparse->new();
      }
    $timevals{$time}->add_data($val);
  }

$exp_cnt = $timevals{(keys %timevals)[0]}->count();

print <<EOH;
# This file was generated by $0
# cmdline: $cmdline
# in dir: $pwd
# Format: <time> <meaningless number> <avg percent time action in actmap>
EOH

foreach $time (sort {$a <=> $b} (keys %timevals))
  {
    warn "Mismatched counts for time $time" if ($timevals{$time}->count() != $exp_cnt);
    print "$time 12345678 " . $timevals{$time}->mean() . "\n";
  }
