#!/usr/local/bin/perl -w

# This script takes policy eval files and averages the values

$usage = "$0 <list of policy eval files>";

die $usage if not (@ARGV);

use Statistics::Descriptive;

$cmdline = join ' ', ($0, @ARGV);
$pwd = `pwd`;
chomp $pwd;

while (<>)
  {
    s/\#.*//;
    next if (/^\s*$/);
    ($time, $val) = split;
    if (not exists $timevals{$time})
      {
        $timevals{$time} = Statistics::Descriptive::Sparse->new();
      }
    $timevals{$time}->add_data($val);
  }

# note that we use the time value from above
$exp_cnt = $timevals{$time}->count();

print <<EOH;
# This file was generated by avg_policy_eval.pl
# cmdline: $cmdline
# in dir: $pwd
# Format: <time> <avg policy value>
EOH

foreach $time (sort {$a <=> $b} (keys %timevals))
  {
    die "Mismatched counts for time $time" if ($timevals{$time}->count() != $exp_cnt);
    print "$time " . $timevals{$time}->mean() . "\n";
  }
