#!/usr/local/bin/perl5 -w

# this script runs experiments with disabled actions for the predator

$usage = "$0 <data dir> <coach_prob> <num trials>";

# set this to 0 to run full interaction experiments
$use_sa = 0;
@sa_num_states = (50, 25, 1);
#$sim_length =   10000;
#$sim_length =   50000;
#$sim_length =  500000;
#$sim_length = 1500000;
$sim_length = 7000000;

use vars qw($basedatadir $enginepid $cspid);

use Cwd qw(abs_path);
use File::Basename;
use File::Copy;

#die "please set SPADESHOME" if (not exists $ENV{"SPADESHOME"});
#$spadesdir = abs_path($ENV{"SPADESHOME"});

$scriptdir = abs_path(dirname($0));
do "$scriptdir/common.pl" || die "Could not use common.pl: $!";

$basedatadir = shift @ARGV;
$coach_prob = shift @ARGV;
$trials = shift @ARGV;
die $usage if (!defined($trials));

# set up the output directory
$expdir = "$basedatadir/run_disable_exp-$coach_prob.$$";
mkdir ($expdir, 0777) || die "Could not make exp dir '$expdir': $!";

print "Using experiment dir: $expdir\n";

# see common.pl
open (EXECUTELOGFH, ">$expdir/execute.log") || die "Could not open execute log: $!";

print EXECUTELOGFH "Run on ".`date`."\n";

# copy some conf files
copy "$basedir/world/predprey.conf", $expdir;
copy "$basedir/predator/predator.conf", $expdir;
copy "$basedir/coach/coach.conf", $expdir;
copy "$basedir/shared/shared.conf", $expdir;


foreach $sa_num_states ($use_sa ? @sa_num_states : (0))  {

  if ($use_sa)
    {
      $sa_dir = "$expdir/5000-$sa_num_states";
      mkdir ($sa_dir, 0777) || die "Could not make sa trial dir '$sa_dir': $!";
    }

  for ($cnt=0; $cnt < $trials; $cnt++) {
    # make the trial dir
    $trialdir = $use_sa ? "$sa_dir/$cnt" : "$expdir/$cnt";
    mkdir ($trialdir, 0777) || die "Could not make trial dir '$trialdir': $!";

    $preddisablefile = "$trialdir/pred-disable.actmap";

    # make the disable map
    chdir $coachdir || die "Could not change to coach dir";
    $cmdline = "$coachdir/coach --file coach.conf --run_mode create_action_map ";
    #$cmdline .= "---cam_in_opt_fn opt4x4-1-1-opt.actmap --cam_in_nonopt_fn opt4x4-1-1-nonopt.actmap ";
    $cmdline .= "--cam_in_opt_fn opt6x6-1-2-opt.actmap --cam_in_nonopt_fn opt6x6-1-2-nonopt.actmap ";
    $cmdline .= "--cam_fraction_non_opt 0.0 --cam_remove_one_opt ";
    $cmdline .= "--cam_out_fn $preddisablefile ";
    $cmdline .= "--action_log_fn coach-create-actions.log ";
    $cmdline .= "--logfile_dir $trialdir ";
    system($cmdline);
    chdir $scriptdir || die "Could not change back to script dir";

    $nocoachtrialdir = "$trialdir/nocoach";
    mkdir ($nocoachtrialdir, 0777) || die "Could not make no coach trial dir '$nocoachtrialdir': $!";

    run_exp($sim_length,
            "--act_mode qlearn --qlearn_disable_map_fn $preddisablefile",
	    "--act_mode qlearn ",
	    " --file $expdir/predprey.conf --use_pred_coach off ",
	    $nocoachtrialdir);

    $coach_shared_cmd_line = "--qlearn_cmpactmap_interval 5000 ";
    $coach_shared_cmd_line .= "--qlearn_cmpactmap_actmap_fn $preddisablefile ";
    $coach_shared_cmd_line .= "--qlearn_action_cmpactmap_interval 10000 ";
    $coach_shared_cmd_line .= "--qlearn_action_cmpactmap_actmap_fn $preddisablefile ";

    if ($use_sa) {
      #foreach $see (qw(on off)) {
      # optq has to see the agent actions
      foreach $see (qw(on)) {
	foreach $alg (qw(optq random)) {
	  $coachtrialdir = "$trialdir/coach-$alg-$see";
	  mkdir ($coachtrialdir, 0777) || die "Could not make coach trial dir '$coachtrialdir': $!";

	  run_exp($sim_length,
                  "--act_mode qlearn_csa_simple --qlearn_disable_map_fn $preddisablefile --qlearn_wcoach_coach_prob $coach_prob --saadvice_final_advice_fn $coachtrialdir/pred_final.advice --saar_type complete ",
		  "--act_mode sa_$alg --saadvice_final_advice_fn $coachtrialdir/coach_final.advice --saadvice_num_states $sa_num_states --saadvice_interval 5000 --saar_type complete $coach_shared_cmd_line --qlearn_cmpactmap_out_fn $coachtrialdir/qlearn_compare_actmap.data --qlearn_action_cmpactmap_out_fn $coachtrialdir/qlearn_action_compare_actmap.data",
		  "--file $expdir/predprey.conf --use_pred_coach on --tell_coach_agent_actions $see",
		  $coachtrialdir);

	}
      }
    } else {
      $coachseetrialdir = "$trialdir/coach-see";
      mkdir ($coachseetrialdir, 0777) || die "Could not make coach see trial dir '$coachseetrialdir': $!";

      run_exp($sim_length,
              "--act_mode qlearn_wcoach --qlearn_disable_map_fn $preddisablefile --qlearn_wcoach_coach_prob $coach_prob",
	      "--act_mode qlearn $coach_shared_cmd_line --qlearn_cmpactmap_out_fn $coachseetrialdir/qlearn_compare_actmap.data --qlearn_action_cmpactmap_out_fn $coachseetrialdir/qlearn_action_compare_actmap.data ",
	      "--file $expdir/predprey.conf --use_pred_coach on --tell_coach_agent_actions on",
	      $coachseetrialdir);

      $coachnoseetrialdir = "$trialdir/coach-nosee";
      mkdir ($coachnoseetrialdir, 0777) || die "Could not make coach no see trial dir '$coachnoseetrialdir': $!";

      run_exp($sim_length,
              "--act_mode qlearn_wcoach --qlearn_disable_map_fn $preddisablefile --qlearn_wcoach_coach_prob $coach_prob",
	      "--act_mode qlearn $coach_shared_cmd_line --qlearn_cmpactmap_out_fn $coachnoseetrialdir/qlearn_compare_actmap.data --qlearn_action_cmpactmap_out_fn $coachnoseetrialdir/qlearn_action_compare_actmap.data",
	      "--file $expdir/predprey.conf --use_pred_coach on --tell_coach_agent_actions off",
	      $coachnoseetrialdir);

      $coachnolearntrialdir = "$trialdir/coach-nolearn";
      mkdir ($coachnolearntrialdir, 0777) || die "Could not make coach no see trial dir '$coachnolearntrialdir': $!";

      run_exp($sim_length,
              "--act_mode qlearn_wcoach --qlearn_disable_map_fn $preddisablefile --qlearn_wcoach_coach_prob $coach_prob",
	      "--act_mode qlearn --forbid_all_q_updates on $coach_shared_cmd_line --qlearn_cmpactmap_out_fn $coachnolearntrialdir/qlearn_compare_actmap.data --qlearn_action_cmpactmap_out_fn $coachnolearntrialdir/qlearn_action_compare_actmap.data",
	      "--file $expdir/predprey.conf --use_pred_coach on --tell_coach_agent_actions off",
	      $coachnolearntrialdir);

      $coachbadadvtrialdir = "$trialdir/coach-badadvice";
      mkdir ($coachbadadvtrialdir, 0777) || die "Could not make coach bad advice trial dir '$coachbadadvtrialdir': $!";
      run_exp($sim_length,
              "--act_mode qlearn_wcoach --qlearn_disable_map_fn $preddisablefile --qlearn_wcoach_coach_prob $coach_prob",
	      "--act_mode actmap --fixed_actmap_1_fn $preddisablefile --fixed_actmap_2_fn opt6x6-1-2-opt.actmap",
	      "--file $expdir/predprey.conf --use_pred_coach on --tell_coach_agent_actions off",
	      $coachbadadvtrialdir);
    }

    sleep 1;
  }
}

close (EXECUTELOGFH);

