#!/usr/local/bin/perl5 -w

# this script runs experiments with disabled actions for the predator
# and limited memory effects on the predator

$usage = "$0 <data dir> <num trials>";

@sa_num_states = (50, 25, 1);
@sa_fifo_sizes = (1000, 5000, 10000, 15000);

#$sim_length =   10000;
#$sim_length =   50000;
#$sim_length =  500000;
#$sim_length = 1500000;
$sim_length = 7000000;

use vars qw($basedatadir $enginepid $cspid);

use Cwd qw(abs_path);
use File::Basename;
use File::Copy;

$scriptdir =  abs_path(dirname($0));

do "$scriptdir/common.pl" || die "Could not use common.pl: $!";

$basedatadir = shift @ARGV;
$trials = shift @ARGV;
die $usage if (!defined($trials));

# set up the output directory
$expdir = "$basedatadir/run_fifo_exp.$$";
mkdir ($expdir, 0777) || die "Could not make exp dir '$expdir': $!";

print "Using experiment dir: $expdir\n";

# see common.pl
open (EXECUTELOGFH, ">$expdir/execute.log") || die "Could not open execute log: $!";

print EXECUTELOGFH "Run on ".`date`."\n";

# copy some conf files
copy "$basedir/world/predprey.conf", $expdir;
copy "$basedir/predator/predator.conf", $expdir;
copy "$basedir/coach/coach.conf", $expdir;
copy "$basedir/shared/shared.conf", $expdir;

for ($cnt=0; $cnt < $trials; $cnt++) {
  # make the trial dir
  $trialdir = "$expdir/$cnt";
  mkdir ($trialdir, 0777) || die "Could not make trial dir '$trialdir': $!";

  $preddisablefile = "$trialdir/pred-disable.actmap";

  # make the disable map
  chdir $coachdir || die "Could not change to coach dir";
  $cmdline = "$coachdir/coach --file coach.conf --run_mode create_action_map ";
  #$cmdline .= "--cam_in_opt_fn opt4x4-1-1-opt.actmap --cam_in_nonopt_fn opt4x4-1-1-nonopt.actmap ";
  $cmdline .= "--cam_in_opt_fn opt6x6-1-2-opt.actmap --cam_in_nonopt_fn opt6x6-1-2-nonopt.actmap ";
  $cmdline .= "--cam_fraction_non_opt 0.0 --cam_remove_one_opt ";
  $cmdline .= "--cam_out_fn $preddisablefile ";
  $cmdline .= "--action_log_fn coach-create-actions.log ";
  $cmdline .= "--logfile_dir $trialdir ";
  system($cmdline);
  chdir $scriptdir || die "Could not change back to script dir";

  $nocoachtrialdir = "$trialdir/nocoach";
  mkdir ($nocoachtrialdir, 0777) || die "Could not make no coach trial dir '$nocoachtrialdir': $!";

  run_exp($sim_length,
          "--act_mode qlearn --qlearn_disable_map_fn $preddisablefile",
	  "--act_mode qlearn ",
	  "--file $expdir/predprey.conf --use_pred_coach off ",
	  $nocoachtrialdir);

  foreach $fifo_size (@sa_fifo_sizes) {
    foreach $sa_num_states (@sa_num_states)  {
      foreach $alg (qw(optq random)) {
	# optq has to see the agent actions
	foreach $see (qw(on)) {
	  $coachtrialdir = "$trialdir/fifo${fifo_size}-5000-$sa_num_states-$alg-$see";
	  mkdir ($coachtrialdir, 0777) || die "Could not make coach trial dir '$coachtrialdir': $!";

	  run_exp($sim_length,
                  "--act_mode qlearn_csa_simple --qlearn_disable_map_fn $preddisablefile --saadvice_final_advice_fn $coachtrialdir/pred_final.advice --saar_type fifo --saar_fifo_limit $fifo_size ",
		  "--act_mode sa_$alg --saadvice_final_advice_fn $coachtrialdir/coach_final.advice --saadvice_num_states $sa_num_states --saadvice_interval 5000 --saar_type fifo --saar_fifo_limit $fifo_size ",
		  "--file $expdir/predprey.conf --use_pred_coach on --tell_coach_agent_actions $see",
		  $coachtrialdir);

	  sleep 1;
	}
      }
    }
  }
}

close (EXECUTELOGFH);

