/* A behavior to provide advice to execute a fixed policy */

#include <algorithm>
#include "AdviseFixedByActMapBehavior.hpp"
#include "Logger.hpp"
#include "random.hpp"
#include "utility.hpp"

using namespace spades;
using namespace spadesalib;

AdviseFixedByActMapBehavior::AdviseFixedByActMapBehavior(AgentCommInterface* pacs)
  : Behavior(pacs), v_actmap()
{
}


AdviseFixedByActMapBehavior::~AdviseFixedByActMapBehavior()
{
  std::for_each(v_actmap.begin(), v_actmap.end(), spades::deleteptr<ActionSubsetMap>());
  v_actmap.clear();
}

  
void
AdviseFixedByActMapBehavior::respondSense(Behavior::SenseType st)
{
  switch (st)
    {
    case ST_None:
      errorlog << "I should get ST_None" << ende;
      break;
    case ST_TimeNotify:
      //do nothing, we never request time notifies
      break;
    case ST_State:
      //do nothing, wait for an explicit request by the agent
      break;
    case ST_Comm:
      if (getCommType() == CT_RequestAction)
	{
	  unsigned sidx = getCurrState().flatVal();
	  bool sent_advice = false;
	  int idx = 0;
	  for (VActMap::iterator iter = v_actmap.begin();
	       iter != v_actmap.end();
	       iter++, idx++)
	    {
	      int count = (*iter)->count(sidx);
	      if (count != 0)
		{
		  basic_move_t m = (basic_move_t)(*iter)->getElement(sidx, int_random(count));
		  actionlog(50) << "AdviseFixedActMapBehavior: on state " << sidx
				<< " with map " << idx
				<< " of action " << m
				<< ende;
		  sendAdvice(m);
		  sent_advice = true;
		  break;
		}
	    }

	  if (!sent_advice)
	    errorlog << "AdviseFixedByActMapBehavior: state not in any map: " << getCurrState() << ende;
	}
      else
	{
	  errorlog << "I don't handle comms other than CT_RequestAction: " << getCommType() << ende;
	}
      break;
    default:
      errorlog << "I don't understand sense type: " << st << ende;
      break;
    }
}


bool
AdviseFixedByActMapBehavior::initialize()
{
  /* in the furture, we may want to calculate a rotation in order to handle multiple agents
     executing the "same" policy */
  return true;
}

