/* a virtual base class behavior for a coach to coach in a state/action framework
   The bandwidth restriction are the primary thing that this adds to the base q learner */

#ifndef _BASE_COACH_SA_BEHAVIOR_H_
#define _BASE_COACH_SA_BEHAVIOR_H_

#include <string>
#include "BaseQLearnBehavior.hpp"
#include "QTable.hpp"
#include "ActionSubsetMap.hpp"
#include "SAAdviceRepository.hpp"

//The behavior takes over responsibility for the memory here
class BaseCoachSABehavior : public BaseQLearnBehavior
{
public:
  //in most other cases, I avoid having the behaviors read from the global parameters
  // I am making an exception here for what the coach bandwidth restrictions are
  BaseCoachSABehavior(spadesalib::AgentCommInterface* pacs,
		      float gamma,
		      explore_t exp, float alpha_explore, float boltzmann_constant,
		      const char* enable_map_fn);
  ~BaseCoachSABehavior();

  void setSAAR(SAAdviceRepository* p);
  
  void setFinalAdviceFN(std::string fn) { final_advice_fn = fn; }

  void respondSenseQLearn(Behavior::SenseType st);
  virtual bool initialize();
  virtual void respondSenseSA(Behavior::SenseType st) = 0;

  //this is the function which is called when it is time for more advice
  virtual void getStateActionAdvice(int max_num_to_send, vec_sa_advice_t& v) = 0;

protected:
  const SAAdviceRepository* getAdviceRepository() const;
  
private:
  ActionSubsetMap enable_map;

  SAAdviceRepository* psaar;

  std::string final_advice_fn;
  
  spades::SimTime last_advice_send;
};


#endif
