/* A behavior to execute coach q learning (my new technique) */

#include "CoachQLearnBehavior.hpp"
#include "Logger.hpp"
#include "State.hpp"
#include "Policy.hpp"
#include "CoachParam.hpp"

using namespace std;
using namespace spades;
using namespace spadesalib;

CoachQLearnBehavior::CoachQLearnBehavior(AgentCommInterface* pacs,
					 float gamma,
					 explore_t exp,
					 float alpha_explore, float boltzmann_constant,
					 const char* enable_map_fn)
  : BaseQLearnBehavior(pacs, gamma, exp, alpha_explore, boltzmann_constant),
    enable_map(enable_map_fn), advised_last_cycle(false)
{
  qtable.disableAll();
  int count = qtable.enableSet(enable_map);
  actionlog(110) << "CoachQLearnBehavior: " << count << " actions enabled" << ende;
}


CoachQLearnBehavior::~CoachQLearnBehavior()
{
}

  
void
CoachQLearnBehavior::respondSenseQLearn(Behavior::SenseType st)
{
  switch (st)
    {
    case ST_None:
      errorlog << "I don't know what to do with ST_None" << ende;
      break;
    case ST_TimeNotify:
      //do nothing, but we should never get here
      break;
    case ST_State:
      //update our Q-values, but don't send actions
      //we have to do this here because the reward value will get cleared by Behavior
      if (advised_last_cycle)
	coachUpdateQValue();
      stateSenseUpdate();
      advised_last_cycle = false;
      break;
    case ST_Comm:
      if (getCommType() == CT_RequestAction)
	sendAdvisedAction();
      else
	errorlog << "I only handle comm of request action: " << getCommType() << ende;
      break;
    default:
      errorlog << "I don't understand st: " << st << ende;
      break;
    }
  
}


bool
CoachQLearnBehavior::initialize()
{
  if (!BaseQLearnBehavior::initialize())
    return false;
  return true;
}


void
CoachQLearnBehavior::coachUpdateQValue()
{
  if (!isLastValid())
    {
      actionlog(20) << "CoachQLearnBehavior::coachUpdateQValue: doing nothing for first time" << ende;
      return;
    }

  actionlog(20) << "CoachQLearnBehavior: updating Q" << ende;
  //SMURF: this is a big hack to fix the entity at 0. I'll fix it later
  if (CoachParam::instance()->getNumPred() > 1)
    errorlog << "There's a big hack in CoachQLearnBehavior to look at pred 0" << ende;
  map_action_recv_t::const_iterator iter = getMapActionRecv().find(0);
  if (iter == getMapActionRecv().end())
    {
      actionlog(20) << "CoachQLearnBehavior: updating Q because I didn't see the agent action" << ende;
      updateQValue();
    }
  else if (iter->second == getLastAction())
    {
      actionlog(20) << "CoachQLearnBehavior: updating Q because actions are the same" << ende;
      updateQValue();
    }
  else
    {
      if (qtable.isEnabled(getLastState().flatVal(), iter->second))
	{
	  actionlog(20) << "CoachQLearnBehavior: for Q update: action diff: mine="
			<< getLastAction() << ", theirs=" << iter->second
			<< "; updating for the action performed"
			<< ende;
	  setLastAction(iter->second);
	  updateQValue();
	}
      else
	{
	  actionlog(20) << "CoachQLearnBehavior: for Q update: action diff: mine="
			<< getLastAction() << ", theirs=" << iter->second
			<< "; disabled, so doing nothing"
			<< ende;
	}
    }
}

void
CoachQLearnBehavior::sendAdvisedAction()
{
  basic_move_t action =
    (basic_move_t)qtable.getAct(exp_type, getCurrState().flatVal(), alpha_explore, boltzmann_constant);
  actionlog(20) << "CoachQLearnBehavior: advising " << action
		<< " for state " << getCurrState().flatVal() << ende;
  sendAdvice(action);
  setLastAction(action);
  actForActionCompare(getCurrState().flatVal(), action);
  advised_last_cycle = true;
}
