/* a state aciton advisoing behavior that picks a set of states to advise about each time */

#include "RandomCoachSABehavior.hpp"
#include "random.hpp"
#include "Logger.hpp"

using namespace spades;
using namespace spadesalib;

RandomCoachSABehavior::RandomCoachSABehavior(AgentCommInterface* pacs,
					     float gamma,
					     explore_t exp, float alpha_explore, float boltzmann_constant,
					     const char* enable_map_fn)
  : BaseCoachSABehavior(pacs, gamma, exp, alpha_explore, boltzmann_constant, enable_map_fn)
{
}

RandomCoachSABehavior::~RandomCoachSABehavior()
{
}

  
bool
RandomCoachSABehavior::initialize()
{
  return true;
}

void
RandomCoachSABehavior::respondSenseSA(Behavior::SenseType st)
{
  //do nothing;
}


//this is the function which is called when it is time for more advice
void
RandomCoachSABehavior::getStateActionAdvice(int max_num_to_send, vec_sa_advice_t& v)
{
  //each time we'll pick a random state, and a random enabled action
  for (int i=0; i<max_num_to_send; i++)
    {
      unsigned sidx = int_random(State::getNumStates());
      //the enable map may have restricted what actions we consider
      basic_move_t m = (basic_move_t)qtable.getActRandom(sidx);
      actionlog(60) << "RandomCoachSABehavior: advising on "
		    << sidx << ' ' << m << ende;
      v.push_back(SAAdvice(sidx, m));
    }		
}


