#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <string>
#include "Logger.hpp"
#include "AgentCommInterface.hpp"
#include "utility.hpp"
#include "CoachParam.hpp"
#include "PPSharedParam.hpp"
#include "CoachTagFunction.hpp"
#include "PPSharedTypes.hpp"
#include "random.hpp"
#include "NullBehavior.hpp"
#include "AdviseFixedBehavior.hpp"
#include "AdviseFixedByActMapBehavior.hpp"
#include "SAAdviseFixedBehavior.hpp"
#include "CoachQLearnBehavior.hpp"
#include "PredPreyMDP.hpp"
#include "ActionSubsetMap.hpp"
#include "RandomCoachSABehavior.hpp"
#include "OptQCoachSABehavior.hpp"
#include "FIFOLimSAAdviceRepository.hpp"

using namespace spades;
using namespace spadesalib;
using namespace std;

/*****************************************************************************************/

//this is static so that the signal handlers can access it
static AgentCommInterface* pacs_interface = NULL;
//static so the run methods can set the time strings
static CoachTagFunction *pTagFunc = NULL;

void init_handler_term();
void init_handler_segv();

/*****************************************************************************************/

class ACSITimeTagger : public TimeTagger
{
public:
  ACSITimeTagger(AgentCommInterface* p) : pacs(p) {}
  std::string getTimeTag() { return toString(pacs->getLatestTime()); }

private:
  AgentCommInterface* pacs;
};

SAAdviceRepository*
getNewSAAR()
{
  switch (CoachParam::instance()->getSAARType()) {
  case SAAR_None:
    errorlog << "I don't know wha to do with SAAR_None" << ende;
    break;
  case SAAR_Complete:
    return new SAAdviceRepository(State::getNumStates());
    break;
  case SAAR_FIFO:
     return new FIFOLimSAAdviceRepository(State::getNumStates(), CoachParam::instance()->getSAARFIFOLimit());
    break;
  default:
    errorlog << "I don't understand saar_type: " << CoachParam::instance()->getSAARType() << ende;
    break;
  }
  return NULL;
}


int
runOnline()
{
  pacs_interface = new AgentCommInterface();
  
  pTagFunc->setTimeTagger(new ACSITimeTagger(pacs_interface));
  
  Behavior* pbeh = NULL;

  actionlog(10) << "Coach started" << ende;

  switch (CoachParam::instance()->getActMode())
    {
    case AM_None:
      errorlog << "I don't know what to do with act mode none " << CoachParam::instance()->getActMode() << ende;
      break;
    case AM_Null:
      pbeh = new NullBehavior(pacs_interface);
      break;
    case AM_Policy:
      pbeh = new AdviseFixedBehavior(pacs_interface, CoachParam::instance()->getFixedPolicyFN().c_str());
      break;
    case AM_ActMap: {
      AdviseFixedByActMapBehavior* p = new AdviseFixedByActMapBehavior(pacs_interface);
      //SMURF: this is ugly, but I need a quick hack
      if (CoachParam::instance()->getFixedActMap1FN() != "")
	{
	  ActionSubsetMap* pactmap =
	    new ActionSubsetMap(CoachParam::instance()->getFixedActMap1FN().c_str());
	  p->addActMap(pactmap);
	}
      if (CoachParam::instance()->getFixedActMap2FN() != "")
	{
	  ActionSubsetMap* pactmap =
	    new ActionSubsetMap(CoachParam::instance()->getFixedActMap2FN().c_str());
	  p->addActMap(pactmap);
	}
      if (CoachParam::instance()->getFixedActMap3FN() != "")
	{
	  ActionSubsetMap* pactmap =
	    new ActionSubsetMap(CoachParam::instance()->getFixedActMap3FN().c_str());
	  p->addActMap(pactmap);
	}
      pbeh = p;
      
    } break;      
    case AM_QLearn: {
      CoachQLearnBehavior* p = 
	new CoachQLearnBehavior(pacs_interface,
				CoachParam::instance()->getGamma(),
				CoachParam::instance()->getQLearnExplore(),
				CoachParam::instance()->getQLearnExploreProb(),
				CoachParam::instance()->getQLearnBoltzmannConstant(),
				CoachParam::instance()->getQLearnEnableMapFN().c_str());
      p->setFinalPolicyFN(CoachParam::instance()->getQLearnFinalPolicyFN());
      p->setFinalQTableFN(CoachParam::instance()->getQLearnFinalQTableFN());
      if (CoachParam::instance()->getQLearnCmpActmapInterval() > 0)
	{
	  ActionSubsetMap *pmap =
	    new ActionSubsetMap(CoachParam::instance()->getQLearnCmpActmapActmapFN().c_str());
	  p->setCompareToActMap(CoachParam::instance()->getQLearnCmpActmapInterval(),
				pmap,
				CoachParam::instance()->getQLearnCmpActmapOutFN());
	}
      if (CoachParam::instance()->getQLearnActionCmpActmapInterval() > 0)
	{
	  ActionSubsetMap *pmap =
	    new ActionSubsetMap(CoachParam::instance()->getQLearnActionCmpActmapActmapFN().c_str());
	  p->setActionCompareToActMap(CoachParam::instance()->getQLearnActionCmpActmapInterval(),
				      pmap,
				      CoachParam::instance()->getQLearnActionCmpActmapOutFN());
	}
      pbeh = p;
    } break;
    case AM_SAPolicy:
      pbeh = new SAAdviseFixedBehavior(pacs_interface, CoachParam::instance()->getFixedPolicyFN().c_str());
      break;
    case AM_SARandom: {
      RandomCoachSABehavior* p =
	new RandomCoachSABehavior(pacs_interface, 
				  CoachParam::instance()->getGamma(),
				  CoachParam::instance()->getQLearnExplore(),
				  CoachParam::instance()->getQLearnExploreProb(),
				  CoachParam::instance()->getQLearnBoltzmannConstant(),
				  CoachParam::instance()->getQLearnEnableMapFN().c_str());
      p->setSAAR(getNewSAAR());
      p->setFinalPolicyFN(CoachParam::instance()->getQLearnFinalPolicyFN());
      p->setFinalQTableFN(CoachParam::instance()->getQLearnFinalQTableFN());
      p->setFinalAdviceFN(CoachParam::instance()->getSAAdviceFinalAdviceFN());
      if (CoachParam::instance()->getQLearnCmpActmapInterval() > 0)
	{
	  ActionSubsetMap *pmap =
	    new ActionSubsetMap(CoachParam::instance()->getQLearnCmpActmapActmapFN().c_str());
	  p->setCompareToActMap(CoachParam::instance()->getQLearnCmpActmapInterval(),
				pmap,
				CoachParam::instance()->getQLearnCmpActmapOutFN());
	}
      pbeh = p;
    } break;
    case AM_SAOptQ: {
      OptQCoachSABehavior* p =
	new OptQCoachSABehavior(pacs_interface, 
				CoachParam::instance()->getGamma(),
				CoachParam::instance()->getQLearnExplore(),
				CoachParam::instance()->getQLearnExploreProb(),
				CoachParam::instance()->getQLearnBoltzmannConstant(),
				CoachParam::instance()->getQLearnEnableMapFN().c_str(),
				CoachParam::instance()->getSAOptQOptQTableFN().c_str());
      p->setSAAR(getNewSAAR());
      p->setFinalPolicyFN(CoachParam::instance()->getQLearnFinalPolicyFN());
      p->setFinalQTableFN(CoachParam::instance()->getQLearnFinalQTableFN());
      p->setFinalAdviceFN(CoachParam::instance()->getSAAdviceFinalAdviceFN());
      if (CoachParam::instance()->getQLearnCmpActmapInterval() > 0)
	{
	  ActionSubsetMap *pmap =
	    new ActionSubsetMap(CoachParam::instance()->getQLearnCmpActmapActmapFN().c_str());
	  p->setCompareToActMap(CoachParam::instance()->getQLearnCmpActmapInterval(),
				pmap,
				CoachParam::instance()->getQLearnCmpActmapOutFN());
	}
      pbeh = p;
    } break;
    default:
      errorlog << "What is act mode? " << CoachParam::instance()->getActMode() << ende;
      break;
    }
  
  
  int res = pbeh->behaviorLoop();
  //int res = mainLoop(pacs_interface);

  actionlog(10) << "Client is exiting after " << res << " messages"
		<< ende;

  delete pbeh;

  //the signal handlers might try to access pacs_interface
  AgentCommInterface* pTmp = pacs_interface;
  pacs_interface = NULL;
  delete pTmp;
  
  return res;
}

int
runCreateMDP()
{
  cout << "Creating an MDP...." << endl;
  
  PredPreyMDP m;
  ofstream out(CoachParam::instance()->getMDPFN().c_str());
  if (!out)
    {
      errorlog << "runCreateMDP: could not open out file '" << CoachParam::instance()->getMDPFN() << "'" << ende;
      return -1;
    }

  //false says no wraparound
  m.createFromCurrentState(false, CoachParam::instance()->getReward());

  cout << "Done, collapsing transitions...." << endl;
  m.collapseTransitions();
  
  cout << "Done, verifying...." << endl;
  if (!m.verify(true))
    errorlog << "Error in verification!" << ende;
  
  cout << "Done, writing to file...." << endl;

  out << m;
  
  cout << "Finished creating an MDP...." << endl;
  
  return 0;
}

int
runSolveMDP()
{
  cout << "Solving an MDP...." << endl;
  
  MDP m;
  ifstream in(CoachParam::instance()->getMDPFN().c_str());
  ofstream outqt((CoachParam::instance()->getSolveMDPOutFStem() + ".qtable").c_str());;
  ofstream outpolicy((CoachParam::instance()->getSolveMDPOutFStem() + ".policy").c_str());;
  if (!in)
    {
      errorlog << "runSolveMDP: could not open in file '" << CoachParam::instance()->getMDPFN() << "'" << ende;
      return -1;
    }
  if (!outqt)
    {
      errorlog << "runSolveMDP: could not open out qtable file '"
	       << (CoachParam::instance()->getSolveMDPOutFStem() + ".qtable") << "'" << ende;
      return -1;
    }
  if (!outpolicy)
    {
      errorlog << "runSolveMDP: could not open out policy file '"
	       << (CoachParam::instance()->getSolveMDPOutFStem() + ".policy") << "'" << ende;
      return -1;
    }

  in >> m;
  if (in.fail())
    {
      errorlog << "runSolveMDP: failed MDP reading" << ende;
      return -1;
    }
  in.close();

  cout << "MDP read finished..." << endl;
  
  QTableFlat qtable(m.getNumStates(), m.getNumActions(), CoachParam::instance()->getGamma());

  cout << "QTable constructed..." << endl;

  int count = m.solveByQTable(qtable, true);

  cout << "Solution converged..." << endl;

  outqt << qtable;
  outqt.close();

  cout << "QTable written..." << endl;

  Policy p(State::getNumStates());
  p.extractFromQTable(&qtable);
  outpolicy << p;
  outpolicy.close();
  
  cout << "Finished solving MDP after " << count << " iterations" << endl;
  
  return 0;
}

int
runExtractActionMap()
{
  cout << "Extracting a disable action map from a qtable" << endl;
  
  QTableFlat qt(CoachParam::instance()->getExtractMapInQTableFN().c_str());
  
  cout << "Read QTable" << endl;

  ofstream outopt((CoachParam::instance()->getExtractMapOutFStem() + "-opt.actmap").c_str());
  ofstream outnonopt((CoachParam::instance()->getExtractMapOutFStem() + "-nonopt.actmap").c_str());
  if (!outopt)
    {
      errorlog << "runExtractDisableMap: could not open out opt file '"
	       << (CoachParam::instance()->getExtractMapOutFStem() + "-opt.actmap") << "'" << ende;
      return -1;
    }
  if (!outnonopt)
    {
      errorlog << "runExtractDisableMap: could not open out nonoptfile '"
	       << (CoachParam::instance()->getExtractMapOutFStem() + "-nonopt.actmap") << "'" << ende;
      return -1;
    }

  int count = qt.disableNonApproxMax(CoachParam::instance()->getMaxPercForQTable());
  actionlog(30) << "disabled " << count << " non-opt actions" << ende;
  cout << "Disabled actions" << endl;

  ActionSubsetMap m;
  m.extractDisabledFromQTable(qt);

  outnonopt << m;

  cout << "Wrote nonopt" << endl;

  m.invert(NUM_BASIC_MOVE_T);

  outopt << m;

  cout << "Wrote opt" << endl;

  cout << "Done" << endl;

  return 0;
}

int
runCreateActionMap()
{
  cout << "Creating an action map from others" << endl;

  int count;
  ActionSubsetMap asm_opt(CoachParam::instance()->getCAMInOptFN().c_str());
  ActionSubsetMap asm_nonopt(CoachParam::instance()->getCAMInNonoptFN().c_str());
  ofstream out(CoachParam::instance()->getCAMOutFN().c_str());
  if (!out)
    {
      errorlog << "CreateActionMap: could not open out file '"
	       << CoachParam::instance()->getCAMOutFN() << "'" << ende;
      return -1;
    }

  //now do the stuff with the asm_opt
  if (CoachParam::instance()->getCAMRemoveOneOpt())
    {
      count = asm_opt.removeOnePerState();
      actionlog(30) << "Removed " << count << " from opt action, one per state" << ende;
    }
    
  //now do the stuff with the asm_nonopt
  count = asm_nonopt.removeRandom(1.0 - CoachParam::instance()->getCAMFractionNonOpt());
  actionlog(30) << "Randomly removed " << count << " from non opt action" << ende;
  
  //now merge and write it out!
  asm_opt.unionWith(asm_nonopt);

  out << asm_opt;

  cout << "Done" << endl;

  return 0;
}

/* pretty print policy */
int
runPPPolicy()
{
  Policy p(CoachParam::instance()->getFixedPolicyFN().c_str());

  //p.printPretty(cout);
  errorlog << "I can not pretty print a policy right now! unimple!" << ende;

  cout << endl;

  return 0;
}

/* Compare an action map to a policy */
int
runCmpActionMap()
{
  Policy p(CoachParam::instance()->getCmpActmapPolicyFN().c_str());
  ActionSubsetMap m(CoachParam::instance()->getCmpActmapActmapFN().c_str());
  int incnt, outcnt, skipcnt;

  CaptureStateFilter filter(false);
  m.compareToPolicy(p, &filter, &incnt, &outcnt, &skipcnt);
  
  cout << "In the map: " << incnt << endl;
  cout << "Not in the map: " << outcnt << endl;
  cout << "Skipped: " << skipcnt << endl;
  
  return 0;
}

//SMURF: I haven't really parameterixed this yet because I need it for a quick hack
// I'll fix it later
int
runMakePolicy()
{
  cout << "Creating a policy..." << endl;
  
  char* opt_actmap_fn = "opt6x6-1-2-opt.actmap";
  char* out_fn = "make.policy";
  
  //this makes a policy that is optimal where there is only one optimal action
  // and a random sub-optmal otherwise
  ActionSubsetMap actmap_opt(opt_actmap_fn);
  ActionSubsetMap actmap_nonopt(actmap_opt);
  actmap_nonopt.invert(NUM_BASIC_MOVE_T);
  Policy p(actmap_opt.getNumStates());
  
  for (int s = 0; s<actmap_opt.getNumStates(); s++)
    {
      if (actmap_opt.count(s) == 1 || actmap_opt.count(s) == NUM_BASIC_MOVE_T)
	{
	  //put the single optimal act in the policy
	  p.setAction(s, actmap_opt.getElement(s, 0));
	}
      else
	{
	  //pick a random non-optimal
	  //p.setAction(s, actmap_nonopt.getElement(s, int_random(actmap_nonopt.count(s))));
	  //pick a random action
	  p.setAction(s, int_random(NUM_BASIC_MOVE_T));
	}
      
    }
  

  ofstream out(out_fn);
  out << p;
  out.close();

  return 0;
}

/*****************************************************************************************/

void
testMDP()
{
  MDP m;
  ifstream is("test.mdp");
  is >> m;
  cout << "read fail: " << is.fail() << endl;
  cout << "verify: " << m.verify(true) << endl;
  cout << m;
}

void
testStateFlatVal()
{
  int err_count = 0;
  
  for (unsigned int i=0; i < State::getNumStates(); i++)
    {
      State s(i);
      if (s.flatVal() != i)
	{
	  errorlog << "State mismatch: " << s.flatVal() << ' ' << i << ' ' << s << ende;
	  if (++err_count > 10)
	    break;
	}
      /*
      else
	{
	  cout << "State " << i << " okay" << endl;
	}
      */
    }
}

void
testSAAR()
{
  SAAdviceRepository saar(3);

  saar.addAdvice(SAAdvice(0, BM_East));
  saar.addAdvice(SAAdvice(1, BM_South));
  saar.addAdvice(SAAdvice(2, BM_West));

  cout << saar;
}

void
testFIFOSAAR()
{
  FIFOLimSAAdviceRepository saar(10, 5);

  saar.addAdvice(SAAdvice(0, BM_East));
  saar.addAdvice(SAAdvice(2, BM_South));
  saar.addAdvice(SAAdvice(4, BM_West));
  saar.addAdvice(SAAdvice(6, BM_East));
  saar.addAdvice(SAAdvice(8, BM_South));

  cout << "Pass 1:" << endl;
  cout << saar;
  
  saar.addAdvice(SAAdvice(1, BM_East));
  saar.addAdvice(SAAdvice(3, BM_South));

  cout << "Pass 2:" << endl;
  cout << saar;
  
  saar.addAdvice(SAAdvice(5, BM_West));
  saar.addAdvice(SAAdvice(7, BM_East));
  saar.addAdvice(SAAdvice(9, BM_South));

  cout << "Pass 3:" << endl;
  cout << saar;
  
}

void
testFIFOSAARBig()
{
  FIFOLimSAAdviceRepository saar(40000, 5000);
  ofstream out("test_saar.advice");
  
  for (int i=0; i<6500; i++)
    {
      saar.addAdvice(SAAdvice(int_random(40000), BM_South));
    }

  out << saar;
}

void
testArgs(int argc, char** argv)
{
  cout << "argc: " << argc << endl;
  for (int i = 0; i < argc; i++)
    cout << argv[i] << ' ';
  cout << endl;
  cout << "saadvice_num_states: " << CoachParam::instance()->getSAAdviceNumStates() << endl;
}

/*****************************************************************************************/

int main (int argc, char** argv)
{
  CoachParam* pParam = new CoachParam(argc, argv);

  pTagFunc = new CoachTagFunction;

  spades::Logger::instance()->setTagFunction(pTagFunc);
  spades::Logger::instance()->setMaxActionLogLevel(pParam->getActionLogLevel());
  if (pParam->getRunMode() != RM_Online)
    {
      StringReplacer strrep;
      strrep.addReplacement("%D", pParam->getLogfileDir());
      std::string fn(strrep.performReplacements(pParam->getActionLogFN()));
      Logger::instance()->openFiles(fn);
    }

  init_handler_term();
  init_handler_segv();

  spades::seedRandom(pParam->getRandomSeed());

  //testMDP(); return 0;
  //testStateFlatVal(); return 0;
  //testSAAR(); return 0;
  //testFIFOSAAR(); return 0;
  //testFIFOSAARBig(); return 0;
  //testArgs(argc, argv); return 0;
	
  int res;
  switch (CoachParam::instance()->getRunMode())
    {
    case RM_None:
      cout << "No run mode, doing nothing" << endl;
      res = 0;
      break;
    case RM_Online:
      res = runOnline();
      break;
    case RM_CreateMDP:
      res = runCreateMDP();
      break;
    case RM_SolveMDP:
      res = runSolveMDP();
      break;
    case RM_ExtractActionMap:
      res = runExtractActionMap();
      break;
    case RM_CreateActionMap:
      res = runCreateActionMap();
      break;
    case RM_PPPolicy:
      res = runPPPolicy();
      break;
    case RM_CmpActionMap:
      res = runCmpActionMap();
      break;
    case RM_MakePolicy:
      res = runMakePolicy();
      break;
    default:
      errorlog << "What is run mode? " << CoachParam::instance()->getRunMode() << ende;
      res = -1;
    }

  Logger::removeInstance();

  delete pParam;
  pParam = NULL;

  return res;
}


/*****************************************************************************************/

void sigterm_handler(int) 
{
  if (pacs_interface)
    pacs_interface->initiateShutdown();
}

void init_handler_term() 
{
  struct sigaction sigact;
  sigact.sa_flags = 0;

  //This makes sure that the handler is only called once for each signal
  sigact.sa_flags |= SA_RESETHAND;
  
  sigemptyset(&sigact.sa_mask);
  sigact.sa_handler = sigterm_handler; 
  sigaction(SIGINT, &sigact, NULL);
  sigaction(SIGQUIT, &sigact, NULL);
  sigaction(SIGTERM, &sigact, NULL);
}

void sigsegv_handler(int) 
{
  errorlog << "Got a seg fault!" << ende;
  if (pacs_interface)
    pacs_interface->initiateShutdown();
}

void init_handler_segv() 
{
  struct sigaction sigact;
  sigact.sa_flags = 0;

  //This makes sure that the handler is only called once for each signal
  sigact.sa_flags |= SA_RESETHAND;
  
  sigemptyset(&sigact.sa_mask);
  sigact.sa_handler = sigsegv_handler; 
  sigaction(SIGSEGV, &sigact, NULL);
}
