/* Predator parameters */

#ifndef _PRED_PARAM_H_
#define _PRED_PARAM_H_

#include "PPSharedParam.hpp"
#include "PPSharedTypes.hpp"
#include "QTable.hpp" //for explore_t

enum ActMode_t {
  AM_None,
  AM_Random,
  AM_Policy,
  AM_QLearn,
  AM_StrictCoachReq,
  AM_QLearnWCoach,
  AM_QLearnCSASimple
};


class PredParam : public PPSharedParam
{
public:

  PredParam (const int &argc, const char *const *argv);
  PredParam ();
  virtual ~PredParam ();

  static const PredParam* instance() { return s_instance; }
  
private:
  static PredParam* s_instance;
    
protected:
  //virtual void writeConfig ( ostream& o );

  virtual void addAll2Maps ();
  virtual void setDefaultValues ();
  virtual void postReadProcessing();

public:

  ActMode_t getActMode() const { return act_mode; }

  std::string getFixedPolicyFN() const { return fixed_policy_fn; }

  explore_t getQLearnExplore() const { return qlearn_explore; }
  double getQLearnExploreProb() const { return qlearn_explore_prob; }
  double getQLearnBoltzmannConstant() const { return qlearn_boltzmann_constant; }
  std::string getQLearnFinalPolicyFN() const { return qlearn_final_policy_fn; }
  std::string getQLearnFinalQTableFN() const { return qlearn_final_qtable_fn; }
  std::string getQLearnPolicyChangeFN() const { return qlearn_policy_change_fn; }
  std::string getQLearnPolicyEvalFN() const { return qlearn_policy_eval_fn; }
  int getQLearnPolicyEvalInterval() const { return qlearn_policy_eval_interval; }
  int getQLearnPolicyEvalLength() const { return qlearn_policy_eval_length; }
  std::string getQLearnDisableMapFN() const { return qlearn_disable_map_fn; }

  double getQLearnWCoachCoachProb() const { return qlearn_wcoach_coach_prob; }
  bool getQLearnWCoachRandomOnAdvisedDisable() const { return qlearn_wcoach_random_on_advised_disable; }

  
  double getQLearnCSACoachProb() const { return qlearn_csa_coach_prob; }
  std::string getSAAdviceFinalAdviceFN() const { return saadvice_final_advice_fn; }
  
protected:

  ActMode_t act_mode;
  std::string act_mode_string;

  std::string fixed_policy_fn;

  std::string qlearn_explore_string;
  explore_t qlearn_explore;
  double qlearn_explore_prob;
  double qlearn_boltzmann_constant;
  std::string qlearn_final_policy_fn;
  std::string qlearn_final_qtable_fn;
  std::string qlearn_policy_change_fn;
  std::string qlearn_policy_eval_fn;
  int qlearn_policy_eval_interval;
  int qlearn_policy_eval_length;
  std::string qlearn_disable_map_fn;

  double qlearn_wcoach_coach_prob;
  bool qlearn_wcoach_random_on_advised_disable;
  
  double qlearn_csa_coach_prob;
  std::string saadvice_final_advice_fn;
  
};


#endif
