/* Create a sense event which a communication from another agent */

#include <cstring>
#include <strstream>
#include "CreateCommSenseEvent.hpp"
#include "SenseEvent.hpp"
#include "PredPreyWorld.hpp"
#include "Logger.hpp"

using namespace std;
using namespace spades;

CreateCommSenseEvent::CreateCommSenseEvent (spades::SimTime t, spades::AgentID a,
					    entityid_t sender, const char* m)
  : CreateSenseEvent(t, a), sender(sender)
{
  int len = strlen(m);
  msg = new char[len+1];
  strcpy(msg, m);
}

CreateCommSenseEvent::~CreateCommSenseEvent ()
{
  if (msg) delete msg;
}


void
CreateCommSenseEvent::print (std::ostream & o) const
{
  o << "CreateCommSenseEvent(t:" << getTime()
    << ",a:" << getAgent()
    << ",msg:" << msg
    << ")";
}


//from CreateSenseEvent
spades::SenseEvent*
CreateCommSenseEvent::createSense(spades::WorldModel* p)
{
  //Don't actually need to access the world
  //PredPreyWorld* pPP = (PredPreyWorld*)p;

  //No sense latency!
  SimTime arrive_time = getTime();
  
  actionlog(150) << "Creating a CommSenseEvent"
		 << "(ts:" << getTime()
		 << ", ta:" << arrive_time
		 << ", a: " << getAgent()
		 << ", msg: " << msg
		 << ")"
		 << ende;

  ostrstream o;
  o << "(comm " << sender << " \"" << msg << "\")";

  o << ends;

  SenseEvent* pE =
    new SenseEvent(TT_Regular, getTime(), arrive_time, getAgent());

  DataArray d(o);
  pE->setData(d);
  
  //note that we do not unfreeze ostrstream since DataArray took it

  return pE;
}

