/* This class is the basic info for an entity in the world, i.e. a agent who
   has a position in the world
   This will not include the coach */

#ifndef _ENTITY_H_
#define _ENTITY_H_

#include <iostream>
#include "PPSharedTypes.hpp"


/*****************************************************************************/

class Entity 
{
public:
  Entity(entityid_t i = -1, Location l = Location(), basic_move_t m = BM_None)
    : id(i), pos(l), next_move(m) {}
  ~Entity() {}

  entityid_t getID() const { return id; }
  void setID(entityid_t i) { id = i; }

  const Location& getLoc() const { return pos; }
  void  setLoc(Location l) { pos = l; }

  void setPosRandom() { pos.setRandom(); }

  basic_move_t getNextMove() const { return next_move; }
  void setNextMove(basic_move_t m) { next_move = m; }
  void clearMove() { next_move = BM_None; }
  
  //Note: this also sets the move to None
  void applyMove();

  bool isSameLocation(const Entity& e) const
  { return pos == e.pos; }
  
  friend std::ostream & operator << (std::ostream & o, const Entity & e)
  { o << "Entity(id:" << e.id << ", p:" << e.pos << ")"; return o; }
  //this is used in creating sensations
  void printCompact(std::ostream& o)
  { o << '(' << id << ' ' << pos.x << ' ' << pos.y << ')'; }
  
private:

  entityid_t id;

  Location pos;

  basic_move_t next_move;
};



#endif
