/* -*- Mode: C++ -*- */

#include "PredPreyParam.hpp"
#include "Logger.hpp"
#include "PPSharedTypes.hpp"

using namespace spades;

PredPreyParam* PredPreyParam::s_instance = NULL;

PredPreyParam::PredPreyParam (const int &argc, const char *const *argv)
  :EngineParam ()
{
  if (s_instance)
    errorlog << "Should only be one instance of PredPreyParam" << ende;
  s_instance = this;

  getOptions (argc, argv);
}

PredPreyParam::~PredPreyParam ()
{
  s_instance = NULL;
}


void
PredPreyParam::addAll2Maps ()
{
  EngineParam::addAll2Maps ();

  add2Maps ("x_size", &x_size);
  add2Maps ("y_size", &y_size);

  add2Maps ("discrete_step_size", &discrete_step_size);

  add2Maps ("simulation_length", &simulation_length );
  add2Maps ("num_comm_servers_wanted", &num_comm_servers_wanted );

  add2Maps ("pred_name", &pred_name);
  add2Maps ("prey_name", &prey_name);
  add2Maps ("pred_coach_name", &pred_coach_name);
  add2Maps ("prey_coach_name", &prey_coach_name);

  add2Maps ("num_pred", &num_pred);
  add2Maps ("num_prey", &num_prey);
  add2Maps ("use_pred_coach", &use_pred_coach);
  add2Maps ("use_prey_coach", &use_prey_coach);

  add2Maps ("reward", &reward);

  add2Maps ("fix_prey_locations", &fix_prey_locations);

  add2Maps ("wraparound", &wraparound);

  add2Maps ("tell_coach_agent_actions", &tell_coach_agent_actions);

}

void
PredPreyParam::setDefaultValues ()
{
  EngineParam::setDefaultValues ();

  x_size = 20;
  y_size = 20;

  discrete_step_size = 10;
  
  simulation_length = 10000;
  num_comm_servers_wanted = 1;

  pred_name = "pred";
  prey_name = "prey";
  pred_coach_name = "";
  prey_coach_name = "";

  num_pred = 0;
  num_prey = 0;
  use_pred_coach = false;
  use_prey_coach = false;

  reward = 10;

  fix_prey_locations = false;

  wraparound = true;

  tell_coach_agent_actions = true;

}

void
PredPreyParam::postReadProcessing()
{
  EngineParam::postReadProcessing();

  total_size = x_size * y_size;

  Location::setSizes(x_size, y_size);
    
}

