/* a simple class to capture the world state at a given time */

#include <strstream>
#include <cctype>
#include <string>
#include <curses.h>
#include "WorldState.hpp"

using namespace std;

/****************************************************************************/

int WorldState::x_size = 10;
int WorldState::y_size = 10;

/****************************************************************************/
bool
skip_white_space(istream& in)
{
  char c;

  for(;;)
    {
      in.get(c);
      if (!in) return false;
      if (!isspace(c))
	{
	  in.putback(c);
	  return true; 
	} 
    } 
} 
/****************************************************************************/

istream&
operator>>(istream& in, EntityInfo& e)
{
  char ch;
  if (!skip_white_space(in))
    return in;
  in.get(ch);
  if (ch != '(')
    {
      in.putback(ch);
      in.setstate(ios::failbit);
      return in;
    }
  in >> e.num >> e.x_pos >> e.y_pos;
  if (!in)
    return in;
  if (!skip_white_space(in))
    return in;
  in.get(ch);
  if (ch != ')')
    {
      in.putback(ch);
      in.setstate(ios::failbit);
      return in;
    }
  
  return in;
}


/****************************************************************************/

WorldState::WorldState()
  : time(-1), vPred(), vPrey()
{
}

WorldState::~WorldState()
{
}

void
WorldState::clear()
{
  time = -1;
  vPred.clear();
  vPrey.clear();
}

/* note that we flip this so that positive y is up */
void
WorldState::drawOnNCurses()
{
  ostrstream o;
  //first the time line
  o << "Time: " << time << ends;
  mvaddstr(0, 0, o.str());
  o.freeze();

  //where the field actually starts
  int y_offset = 1;
  int x_offset = 0;
  
  //now set the field to be all '.'
  for (int y=0; y<y_size; y++)
    {
      move(y+y_offset, x_offset);
      for (int x=0; x<x_size; x++)
	{
	  addch( ((chtype)'.') | A_NORMAL);
	}
    }

  vector<EntityInfo>::iterator iter;
  //Now draw the predators
  for (iter = vPred.begin(); iter != vPred.end(); iter++)
    {
      mvaddch((y_size - 1 - iter->y_pos) + y_offset, iter->x_pos + x_offset,
	      ((chtype)'0' + iter->num) | A_NORMAL);
    }

  //Now draw the prey. Note we do this in reverse video
  for (iter = vPrey.begin(); iter != vPrey.end(); iter++)
    {
      mvaddch((y_size - 1 - iter->y_pos) + y_offset, iter->x_pos + x_offset,
	      ((chtype)'0' + iter->num) | A_REVERSE);
    }

  //move the cursor to the upper left to get it out of the way
  move(0,0);
}


istream&
operator>>(istream& in, WorldState& ws)
{
  char ch;
  string str;

  ws.clear();
  
  in >> ws.time;
  if (!in) return in;
  if (!skip_white_space(in)) return in;

  in.get(ch);
  if (ch != ':')
    {
      in.putback(ch);
      in.setstate(ios::failbit);
      return in;
    }

  in >> str;
  if (str != "(pred")
    {
      in.putback(ch);
      in.setstate(ios::failbit);
      return in;
    }

  while (in)
    {
      EntityInfo e;
      in >> e;
      if (in)
	{
	  ws.vPred.push_back(e);
	}
    }
  in.clear();

  in.get(ch);
  if (ch != ')')
    {
      in.putback(ch);
      in.setstate(ios::failbit);
      return in;
    }

  in >> str;
  if (str != "(prey")
    {
      in.putback(ch);
      in.setstate(ios::failbit);
      return in;
    }

  while (in)
    {
      EntityInfo e;
      in >> e;
      if (in)
	{
	  ws.vPrey.push_back(e);
	}
    }
  in.clear();

  in.get(ch);
  if (ch != ')')
    {
      in.putback(ch);
      in.setstate(ios::failbit);
      return in;
    }


  return in;
}


//static member
bool
WorldState::readHeaderInfo(istream& in)
{
  in >> x_size >> y_size;
  if (!in)
    return false;
  return true;
}

