/* a behavior based on BaseQLearnCSABehavior that uses advice from the coach
   with a given probability if it exists, and follows the given exploration mode
   otherwise */

#include "SimpleQLearnCSABehavior.hpp"
#include "random.hpp"
#include "Logger.hpp"

using namespace std;
using namespace spades;
using namespace spadesalib;

SimpleQLearnCSABehavior::SimpleQLearnCSABehavior(AgentCommInterface* pacs, float gamma,
						 explore_t exp,
						 float alpha_explore, float boltzmann_constant,
						 float coach_prob, const char* disable_map_fn)
  : BaseQLearnCSABehavior(pacs, gamma, exp, alpha_explore, boltzmann_constant),
    coach_prob(coach_prob), disable_map(disable_map_fn)
{
  qtable.disableSet(disable_map);
}


SimpleQLearnCSABehavior::~SimpleQLearnCSABehavior()
{
}

  
void
SimpleQLearnCSABehavior::respondSenseCSA(Behavior::SenseType st)
{
  switch (st)
    {
    case ST_None:
      errorlog << "I shouldn't get ST_None here" << ende;
      break;
    case ST_TimeNotify:
      //Here's where we do the real stuff!
      respondWithAction();
      break;
    case ST_State:
      //just set a time notify so that if the coach wants to tell us something about now, it can
      // the 8 is a hack based on the idea that the cycles are 10 sim steps long. It just makes sure
      // that the coach can send stuff to us if it wants to 
      pacs->send(new ToCSMessage_RequestTimeNotify(pacs->getLatestTime() + 8));
      break;
    case ST_Comm:
      if (getCommType() != CT_StateActionAdvice)
	errorlog << "SimpleQLearnCSABehavior: ST_StateActionAdvice is the only type handled" << ende;
      // the information is already incorporated in BaseQLearnCSABehavior
      break;
    default:
      errorlog << "I don't understand the sense type: " << st << ende;
      break;
    }
  
}

void
SimpleQLearnCSABehavior::respondWithAction()
{
  unsigned state_idx = getCurrState().flatVal();
  basic_move_t action;
  bool have_advice = !getSAAR()->getStateAdvice(state_idx).empty();
  
  if (have_advice && prob_random() < coach_prob)
    {
      /* we're going to try to do what the coach says */
      //find the first action in the list that we can do
      if (getSAAR()->getFirstEnabled(state_idx, &qtable, &((int)action)))
	{
	  actionlog(30) << "SimpleQLearnCSABehavior: " << state_idx
			<< " I am obeying coach: "
			<< action << ende;
	}
      else
	{
	  actionlog(30) << "SimpleQLearnCSABehavior: " << state_idx
			<< " can't follow coach advice!" << ende;
	  action = (basic_move_t)qtable.getActRandom(state_idx);
	}
    }
  else
    {
      actionlog(30) << "SimpleQLearnCSABehavior: " << state_idx
		    << " using the prevailing exploitation/exploration mode: "
		    << state_idx << ' '
		    << exp_type << " (have advice = " << have_advice << ")" << ende;
      action = (basic_move_t)qtable.getAct(exp_type, state_idx, alpha_explore, boltzmann_constant);
    }
  actionlog(20) << "SimpleQLearnCSABehavior: sending action: " << action << ende;
  sendMove(action);
  setLastAction(action);
  
}


bool
SimpleQLearnCSABehavior::initialize()
{
  if (!BaseQLearnCSABehavior::initialize())
    return false;
  return true;
}


